# jwt

[![Build Status][build-img]][build-url]
[![GoDoc][doc-img]][doc-url]
[![Go Report Card][reportcard-img]][reportcard-url]
[![Coverage][coverage-img]][coverage-url]

JSON Web Tokens for Go

## Features

* Simple API.
* Optimized for speed.
* Dependency-free.

## Install

Go version 1.13

```
go get github.com/cristalhq/jwt/v3
```

## Example

```go
// 1. create a signer & a verifier
key := []byte(`secret`)
signer, err := jwt.NewSignerHS(jwt.HS256, key)
checkErr(err)
verifier, err := jwt.NewVerifierHS(jwt.HS256, key)
checkErr(err)

// 2. create q standard claims
// (you can create your own, see: Example_BuildUserClaims)
claims := &jwt.StandardClaims{
    Audience: []string{"admin"},
    ID:       "random-unique-string",
}

// 3. create a builder
builder := jwt.NewBuilder(signer)

// 4. and build a token
token, err := builder.Build(claims)

// 5. here is your token  :)
var _ []byte = token.Raw() // or just token.String() for string

// 6. parse a token (by example received from a request)
tokenStr := token.String()
newToken, errParse := jwt.ParseString(tokenStr)
checkErr(errParse)

// 7. and verify it's signature
errVerify := verifier.Verify(newToken.Payload(), newToken.Signature())
checkErr(errVerify)

// 8. also you can parse and verify in 1 operation
newToken, err = jwt.ParseAndVerifyString(tokenStr, verifier)
checkErr(err)

// 9. get standard claims
var newClaims jwt.StandardClaims
errClaims := json.Unmarshal(newToken.RawClaims(), &newClaims)
checkErr(errClaims)

// 10. see docs for more methods
```

Also see examples: [this above](https://github.com/cristalhq/jwt/blob/master/example_test.go), [build](https://github.com/cristalhq/jwt/blob/master/example_build_test.go), [parse](https://github.com/cristalhq/jwt/blob/master/example_parse_test.go).

## Documentation

See [these docs](https://godoc.org/github.com/cristalhq/jwt/v3).

## License

[MIT License](LICENSE).

[build-img]: https://github.com/cristalhq/jwt/workflows/build/badge.svg
[build-url]: https://github.com/cristalhq/jwt/actions
[doc-img]: https://godoc.org/github.com/cristalhq/jwt?status.svg
[doc-url]: https://pkg.go.dev/github.com/cristalhq/jwt/v3
[reportcard-img]: https://goreportcard.com/badge/cristalhq/jwt
[reportcard-url]: https://goreportcard.com/report/cristalhq/jwt
[coverage-img]: https://codecov.io/gh/cristalhq/jwt/branch/master/graph/badge.svg
[coverage-url]: https://codecov.io/gh/cristalhq/jwt
